@@ -35,14 +35,16 @@ class InvoiceLine:
                 ['analytic_accounts'])['analytic_accounts']
 
         fields.update(AnalyticAccount.analytic_accounts_fields_get(
-                analytic_accounts_field, fields_names))
+                analytic_accounts_field, fields_names,
+                states=cls.analytic_accounts.states,
+                required_states=Eval('type') == 'line'))
         return fields
 
     @classmethod
-    def default_get(cls, fields, with_rec_name=True):
+    def default_get(cls, fields, with_rec_name=True, with_on_change=True):
         fields = [x for x in fields if not x.startswith('analytic_account_')]
         return super(InvoiceLine, cls).default_get(fields,
-            with_rec_name=with_rec_name)
+            with_rec_name=with_rec_name, with_on_change=with_on_change)
 
     @classmethod
     def read(cls, ids, fields_names=None):
@@ -107,42 +109,52 @@ class InvoiceLine:
         return super(InvoiceLine, cls).create(vlist)
 
     @classmethod
-    def write(cls, lines, vals):
+    def write(cls, *args):
         Selection = Pool().get('analytic_account.account.selection')
-        vals = vals.copy()
-        selection_vals = {}
-        for field in vals.keys():
-            if field.startswith('analytic_account_'):
-                root_id = int(field[len('analytic_account_'):])
-                selection_vals[root_id] = vals[field]
-                del vals[field]
-        if selection_vals:
-            for line in lines:
-                if line.type != 'line':
-                    continue
-                accounts = []
-                if not line.analytic_accounts:
-                    # Create missing selection
-                    with Transaction().set_user(0):
-                            selection, = Selection.create([{}])
-                    cls.write([line], {
-                            'analytic_accounts': selection.id,
+        actions = iter(args)
+        args = []
+        for lines, values in zip(actions, actions):
+            values = values.copy()
+            selection_vals = {}
+            for field, value in values.items():
+                if field.startswith('analytic_account_'):
+                    root_id = int(field[len('analytic_account_'):])
+                    selection_vals[root_id] = value
+                    del values[field]
+            if selection_vals:
+                for line in lines:
+                    if line.type != 'line':
+                        continue
+                    accounts = []
+                    if not line.analytic_accounts:
+                        # Create missing selection
+                        with Transaction().set_user(0):
+                                selection, = Selection.create([{}])
+                        cls.write([line], {
+                                'analytic_accounts': selection.id,
+                                })
+                    for account in line.analytic_accounts.accounts:
+                        if account.root.id in selection_vals:
+                            value = selection_vals[account.root.id]
+                            if value:
+                                accounts.append(value)
+                        else:
+                            accounts.append(account.id)
+                    for account_id in selection_vals.values():
+                        if account_id \
+                                and account_id not in accounts:
+                            accounts.append(account_id)
+                    to_remove = list(
+                        set((a.id for a in line.analytic_accounts.accounts))
+                        - set(accounts))
+                    Selection.write([line.analytic_accounts], {
+                            'accounts': [
+                                ('remove', to_remove),
+                                ('add', accounts),
+                                ],
                             })
-                for account in line.analytic_accounts.accounts:
-                    if account.root.id in selection_vals:
-                        value = selection_vals[account.root.id]
-                        if value:
-                            accounts.append(value)
-                    else:
-                        accounts.append(account.id)
-                for account_id in selection_vals.values():
-                    if account_id \
-                            and account_id not in accounts:
-                        accounts.append(account_id)
-                Selection.write([line.analytic_accounts], {
-                        'accounts': [('set', accounts)],
-                        })
-        super(InvoiceLine, cls).write(lines, vals)
+            args.extend((lines, values))
+        super(InvoiceLine, cls).write(*args)
 
     @classmethod
     def delete(cls, lines):
@@ -193,7 +205,7 @@ class InvoiceLine:
                     vals['credit'] = value['credit']
                     vals['account'] = account.id
                     vals['journal'] = self.invoice.journal.id
-                    vals['date'] = (self.invoice.accounting_date 
+                    vals['date'] = (self.invoice.accounting_date
                         or self.invoice.invoice_date)
                     vals['reference'] = self.invoice.reference
                     vals['party'] = self.invoice.party.id
@@ -224,9 +236,9 @@ class Account(ModelSQL, ModelView):
         return accounts
 
     @classmethod
-    def write(cls, accounts, vals):
+    def write(cls, accounts, values, *args):
         InvoiceLine = Pool().get('account.invoice.line')
-        super(Account, cls).write(accounts, vals)
+        super(Account, cls).write(accounts, values, *args)
         # Restart the cache on the fields_view_get method of
         # account.invoice.line
         InvoiceLine._fields_view_get_cache.clear()
